package com.zhilian.online.uitls;

import com.aliyun.ecs20140526.models.*;
import com.aliyun.tea.TeaException;
import com.aliyun.teautil.models.RuntimeOptions;
import com.zhilian.common.core.domain.Result;
import com.zhilian.online.config.AliyunEcsConfig;
import com.zhilian.online.domain.EcsInstance;
import com.zhilian.online.domain.req.EcsCreateReq;
import com.zhilian.online.domain.req.EcsQueryReq;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;
import org.springframework.validation.annotation.Validated;
import com.aliyun.ecs20140526.Client;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

/**
 * @BelongsProject: smart-cloud-server
 * @BelongsPackage: com.zhilian.online.uitls
 * @Author: LiYuan
 * @CreateTime: 2024-04-12  14:57
 * @Description: 阿里云OpenApi调用工具类
 * @Version: 1.0
 */
@Component
@Slf4j
public class AliyunOpenAPIUtils {

    /**
     * 阿里云ECS客户端
     */
    @Autowired
    private Client aliyunClient;

    @Autowired
    private AliyunEcsConfig aliyunEcsConfig;



    /**
     * 查询实例方法
     *
     * @param: ecsQueryReq
     * @return: List<EcsInstance>
     */
    public List<EcsInstance> queryEcsInstances(EcsQueryReq ecsQueryReq) {
        log.info("开始查询实例{}",ecsQueryReq.getInstanceIds());
        List<EcsInstance> ecsInstances = new ArrayList<>();
        DescribeInstancesResponse describeInstancesResponse = null;

        //将请求参数写入查询请求中
        DescribeInstancesRequest describeInstancesRequest = setQueryReq(ecsQueryReq);

        RuntimeOptions runtime = new com.aliyun.teautil.models.RuntimeOptions();
        try {
            // 复制代码运行请自行打印 API 的返回值
            describeInstancesResponse = aliyunClient.describeInstancesWithOptions(describeInstancesRequest, runtime);
            DescribeInstancesResponseBody body = describeInstancesResponse.getBody();
            DescribeInstancesResponseBody.DescribeInstancesResponseBodyInstances instances = body.getInstances();

            ecsInstances = takeQueryInstances(instances);

        } catch (TeaException error) {
            log.error("code:[{}],message:[{}],data:[{}]", error.getCode(), error.getMessage(), error.getData());
        } catch (Exception aError) {
            TeaException error = new TeaException(aError.getMessage(), aError);
            log.error("code:[{}],message:[{}],data:[{}]", error.getCode(), error.getMessage(), error.getData());
        }

        return ecsInstances;
    }


    /**
     * 查询实例方法
     *
     * @param: ecsQueryReq
     * @return: List<EcsInstance>
     */
    public List<EcsInstance> queryEcsInstancesByPrivateIp(List<String> privateIpAddress) {
        log.info("开始查询实例{}",privateIpAddress);
        List<EcsInstance> ecsInstances = new ArrayList<>();
        DescribeInstancesResponse describeInstancesResponse = null;

        //将请求参数写入查询请求中
        String ipaddress = parseIpListToString(privateIpAddress);
        DescribeInstancesRequest describeInstancesRequest = setQueryReq(ipaddress);

        RuntimeOptions runtime = new com.aliyun.teautil.models.RuntimeOptions();
        try {
            // 复制代码运行请自行打印 API 的返回值
            describeInstancesResponse = aliyunClient.describeInstancesWithOptions(describeInstancesRequest, runtime);
            DescribeInstancesResponseBody body = describeInstancesResponse.getBody();
            DescribeInstancesResponseBody.DescribeInstancesResponseBodyInstances instances = body.getInstances();

            ecsInstances = takeQueryInstances(instances);

        } catch (TeaException error) {
            log.error("code:[{}],message:[{}],data:[{}]", error.getCode(), error.getMessage(), error.getData());
        } catch (Exception aError) {
            TeaException error = new TeaException(aError.getMessage(), aError);
            log.error("code:[{}],message:[{}],data:[{}]", error.getCode(), error.getMessage(), error.getData());
        }

        return ecsInstances;
    }




    /**
     * 查询实例方法
     *
     * @param: instanceIds
     * @return: List<EcsInstance>
     */
    public List<EcsInstance> queryEcsInstances(List<String> instanceIds) {
        log.info("开始查询实例{}",instanceIds);
        List<EcsInstance> ecsInstances = new ArrayList<>();
        DescribeInstancesResponse describeInstancesResponse = null;

        //将请求参数写入查询请求中
        DescribeInstancesRequest describeInstancesRequest = setQueryReq(instanceIds);

        RuntimeOptions runtime = new com.aliyun.teautil.models.RuntimeOptions();
        try {
            // 复制代码运行请自行打印 API 的返回值
            describeInstancesResponse = aliyunClient.describeInstancesWithOptions(describeInstancesRequest, runtime);
            DescribeInstancesResponseBody body = describeInstancesResponse.getBody();
            DescribeInstancesResponseBody.DescribeInstancesResponseBodyInstances instances = body.getInstances();

            ecsInstances = takeQueryInstances(instances);

        } catch (TeaException error) {
            log.error("code:[{}],message:[{}],data:[{}]", error.getCode(), error.getMessage(), error.getData());
        } catch (Exception aError) {
            TeaException error = new TeaException(aError.getMessage(), aError);
            log.error("code:[{}],message:[{}],data:[{}]", error.getCode(), error.getMessage(), error.getData());
        }

        return ecsInstances;
    }


    /**
     * @description: 创建阿里云实例
     * @author: LiYuan
     * @param:
     * @return: void
     **/
    public List<String> createInstance(EcsCreateReq ecsCreateReq) {
        //创建实例请求
        RunInstancesRequest runInstancesRequest = makeRunInstancesRequest(ecsCreateReq);

        RuntimeOptions runtime = new RuntimeOptions();

        List<String> instanceIds = new ArrayList<>();

        try {
            // 复制代码运行请自行打印 API 的返回值
            RunInstancesResponse res = aliyunClient.runInstancesWithOptions(runInstancesRequest, runtime);

            //打印执行结果
            RunInstancesResponseBody body = res.getBody();
            instanceIds = body.getInstanceIdSets().getInstanceIdSet();
            log.info("执行结果:[{}],请求ID:[{}],创建实例ID:[{}]",
                    200 == res.getStatusCode() ? "创建成功" : "创建失败",
                    body.getRequestId(), instanceIds);

        } catch (TeaException error) {
            log.error("code:[{}],message:[{}],data:[{}]",
                    error.getCode(), error.getMessage(), error.getData());
        } catch (Exception aError) {
            TeaException error = new TeaException(aError.getMessage(), aError);
            log.error("code:[{}],message:[{}],data:[{}]",
                    error.getCode(), error.getMessage(), error.getData());
        }


//        List<EcsInstance> ecsInstances = queryEcsInstances(instanceIds);
//        putNodeOfCreatedCache(ecsInstances);

        return instanceIds;
    }

    public Result deleteInstance(String instanceId) {
        Result result = new Result<>();
        DeleteInstancesRequest deleteInstancesRequest = new DeleteInstancesRequest()
                .setRegionId(aliyunEcsConfig.getRegionId())
                .setDryRun(false)
                .setForce(true)
                .setTerminateSubscription(false)
                .setInstanceId(Arrays.asList(instanceId));
        com.aliyun.teautil.models.RuntimeOptions runtime = new com.aliyun.teautil.models.RuntimeOptions();
        try {
            // 复制代码运行请自行打印 API 的返回值
            DeleteInstancesResponse deleteInstancesResponse = aliyunClient.deleteInstancesWithOptions(deleteInstancesRequest, runtime);
            Integer statusCode = deleteInstancesResponse.getStatusCode();
            DeleteInstancesResponseBody body = deleteInstancesResponse.getBody();
            log.info(200 == statusCode ? "删除成功" : "删除失败");
            result.setCode(statusCode);
            result.setData(body);
            result.setMsg(200 == statusCode ? "删除成功" : "删除失败");
        } catch (TeaException error) {
            log.error("code:[{}],message:[{}],data:[{}]", error.getCode(), error.getMessage(), error.getData());
        } catch (Exception _error) {
            TeaException error = new TeaException(_error.getMessage(), _error);
            log.error("code:[{}],message:[{}],data:[{}]", error.getCode(), error.getMessage(), error.getData());
        }
        return result;
    }


    /**
     * @description: 将请求参数写入查询请求中
     * @author: LiYuan
     * @param: EcsQueryReq
     * @return: DescribeInstancesRequest
     **/
    public DescribeInstancesRequest setQueryReq(EcsQueryReq ecsQueryReq) {
        StringBuffer stringBuffer = new StringBuffer();
        stringBuffer.append("[");
        ecsQueryReq.getInstanceIds().forEach(item ->{
           stringBuffer.append("\"" + item + "\",");
        });
        stringBuffer.deleteCharAt(stringBuffer.length()-1);
        stringBuffer.append("]");
        return new DescribeInstancesRequest()
                .setRegionId(aliyunEcsConfig.getRegionId())
                .setInstanceIds(stringBuffer.toString());
//                .setInstanceName(ecsQueryReq.getInstanceName())
//                .setPageNumber(ecsQueryReq.getPageNumber())
//                .setPageSize(ecsQueryReq.getPageSize())
//                .setVpcId(ecsQueryReq.getVpcId())
//                .setVSwitchId(ecsQueryReq.getVSwitchId())
//                .setZoneId(ecsQueryReq.getZoneId())
//                .setInstanceNetworkType(ecsQueryReq.getInstanceNetworkType())
//                .setSecurityGroupId(ecsQueryReq.getSecurityGroupId())
//                .setPrivateIpAddresses(ecsQueryReq.getPrivateIpAddresses())
//                .setPublicIpAddresses(ecsQueryReq.getPublicIpAddresses())
//                .setInstanceName(ecsQueryReq.getInstanceName())
//                .setImageId(ecsQueryReq.getImageId())
//                .setInstanceType(ecsQueryReq.getInstanceType())
//                .setStatus(ecsQueryReq.getStatus());
    }

    /**
     * @description: 将请求参数写入查询请求中
     * @author: LiYuan
     * @param: EcsQueryReq
     * @return: DescribeInstancesRequest
     **/
    public DescribeInstancesRequest setQueryReq(String ipAddresses) {
        return new DescribeInstancesRequest()
                .setRegionId(aliyunEcsConfig.getRegionId())
                .setPrivateIpAddresses(ipAddresses);
    }

    public String parseIpListToString(List<String> ipAddress){
        StringBuffer stringBuffer = new StringBuffer();
        stringBuffer.append("[");
        ipAddress.forEach(item ->{
            stringBuffer.append("\"" + item + "\",");
        });
        stringBuffer.deleteCharAt(stringBuffer.length()-1);
        stringBuffer.append("]");
        return stringBuffer.toString();
    }


    /**
     * @description: 将请求参数写入查询请求中
     * @author: LiYuan
     * @param: EcsQueryReq
     * @return: DescribeInstancesRequest
     **/
    public DescribeInstancesRequest setQueryReq(List<String> instances) {
        StringBuffer stringBuffer = new StringBuffer();
        stringBuffer.append("[");
        instances.forEach(item ->{
            stringBuffer.append("\"" + item + "\",");
        });
        stringBuffer.deleteCharAt(stringBuffer.length()-1);
        stringBuffer.append("]");
        return new DescribeInstancesRequest()
                .setRegionId(aliyunEcsConfig.getRegionId())
                .setInstanceIds(stringBuffer.toString());
//                .setInstanceName(ecsQueryReq.getInstanceName())
//                .setPageNumber(ecsQueryReq.getPageNumber())
//                .setPageSize(ecsQueryReq.getPageSize())
//                .setVpcId(ecsQueryReq.getVpcId())
//                .setVSwitchId(ecsQueryReq.getVSwitchId())
//                .setZoneId(ecsQueryReq.getZoneId())
//                .setInstanceNetworkType(ecsQueryReq.getInstanceNetworkType())
//                .setSecurityGroupId(ecsQueryReq.getSecurityGroupId())
//                .setPrivateIpAddresses(ecsQueryReq.getPrivateIpAddresses())
//                .setPublicIpAddresses(ecsQueryReq.getPublicIpAddresses())
//                .setInstanceName(ecsQueryReq.getInstanceName())
//                .setImageId(ecsQueryReq.getImageId())
//                .setInstanceType(ecsQueryReq.getInstanceType())
//                .setStatus(ecsQueryReq.getStatus());
    }


    /**
     * @description: 取出查询结果
     * @author: LiYuan
     * @param: DescribeInstancesResponseBody.DescribeInstancesResponseBodyInstances
     * @return: List<EcsInstance>
     **/
    public List<EcsInstance> takeQueryInstances(DescribeInstancesResponseBody.DescribeInstancesResponseBodyInstances instances) {
        ArrayList<EcsInstance> ecsInstances = new ArrayList<>();
        //遍历请求结果,取出查询到的Ecs实例
        instances.getInstance().stream().forEach(item -> {
            EcsInstance ecsInstance = EcsInstance.builder()
                    .instanceId(item.getInstanceId())
                    .instanceName(item.getInstanceName())
                    .regionId(aliyunEcsConfig.getRegionId())
                    .status(item.getStatus())
                    .instanceType(item.getInstanceType())
                    .cpu(item.getCpu())
                    .memory(item.getMemory())
                    .localStorageCapacity(item.getLocalStorageCapacity())
                    .osName(item.getOSName())
                    .instanceNetworkType(item.getInstanceNetworkType())
                    .internetMaxBandwidthOut(item.getInternetMaxBandwidthOut())
                    .internetMaxBandwidthIn(item.getInternetMaxBandwidthIn())
                    .publicIpAddress(item.getPublicIpAddress().getIpAddress())
                    .privateIpAddress(item.getVpcAttributes().getPrivateIpAddress().getIpAddress())
                    .vpcId(item.getVpcAttributes().getVpcId())
                    .securityGroupIds(item.getSecurityGroupIds().getSecurityGroupId())
                    .creationTime(item.getCreationTime())
                    .expiredTime(item.getExpiredTime())
                    .recyclable(item.getRecyclable())
                    .build();
            ecsInstances.add(ecsInstance);
        });

        return ecsInstances;
    }

    /**
     * 构建阿里云创建实例请求
     *
     * @param: ecsCreateReq
     * @return: RunInstancesRequest
     */
    public RunInstancesRequest makeRunInstancesRequest(EcsCreateReq ecsCreateReq) {

        //构建磁盘配置
        RunInstancesRequest.RunInstancesRequestSystemDisk systemDisk = new RunInstancesRequest.RunInstancesRequestSystemDisk()
                .setSize(ecsCreateReq.getSystemDiskSize())
                .setCategory(ecsCreateReq.getSystemDickCategory());
        //构建创建实例请求
        RunInstancesRequest runInstancesRequest = new RunInstancesRequest()
                .setRegionId(aliyunEcsConfig.getRegionId())
                .setImageId(ecsCreateReq.getImageId())
                .setInstanceType(ecsCreateReq.getInstanceType())
                .setSecurityGroupId(ecsCreateReq.getSecurityGroupId())
                .setVSwitchId(ecsCreateReq.getVSwitchId())
                .setInstanceName(ecsCreateReq.getInstanceName())
                .setInternetMaxBandwidthIn(ecsCreateReq.getInternetMaxBandwidthIn())
                .setInternetMaxBandwidthOut(ecsCreateReq.getInternetMaxBandwidthOut())
                .setUniqueSuffix(ecsCreateReq.getUniqueSuffix())
                .setPassword(ecsCreateReq.getPassword())
                .setZoneId(ecsCreateReq.getZoneId())
                .setInternetChargeType(ecsCreateReq.getInternetChargeType())
                .setSystemDisk(systemDisk)
                .setAmount(ecsCreateReq.getAmount())
                .setMinAmount(ecsCreateReq.getMinAmount())
                .setPeriodUnit(ecsCreateReq.getPeriodUnit())
                .setPeriod(ecsCreateReq.getPeriod())
                .setInstanceChargeType(ecsCreateReq.getInstanceChargeType())
                .setPrivateIpAddress(ecsCreateReq.getPrivateIpAddress());
        return runInstancesRequest;
    }



}
